from .return_class import AbstractApiClass


class ConversationAndProjectSearchResults(AbstractApiClass):
    """
        A conversation and project search results.

        Args:
            client (ApiClient): An authenticated API Client instance
            deploymentConversationId (str): The unique identifier of the deployment conversation.
            name (str): The name of the deployment conversation.
            chatllmProjectId (str): The chatllm project id associated with the deployment conversation.
            chatllmProjectName (str): The name of the chatllm project associated with the deployment conversation.
            createdAt (str): The timestamp at which the deployment conversation was created.
            lastEventCreatedAt (str): The timestamp at which the most recent corresponding deployment conversation event was created at.
            conversationType (str): The type of the conversation, which depicts the application it caters to.
            deploymentId (str): The deployment id associated with the deployment conversation.
            externalApplicationId (str): The external application id associated with the deployment conversation.
    """

    def __init__(self, client, deploymentConversationId=None, name=None, chatllmProjectId=None, chatllmProjectName=None, createdAt=None, lastEventCreatedAt=None, conversationType=None, deploymentId=None, externalApplicationId=None):
        super().__init__(client, None)
        self.deployment_conversation_id = deploymentConversationId
        self.name = name
        self.chatllm_project_id = chatllmProjectId
        self.chatllm_project_name = chatllmProjectName
        self.created_at = createdAt
        self.last_event_created_at = lastEventCreatedAt
        self.conversation_type = conversationType
        self.deployment_id = deploymentId
        self.external_application_id = externalApplicationId
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'deployment_conversation_id': repr(self.deployment_conversation_id), f'name': repr(self.name), f'chatllm_project_id': repr(self.chatllm_project_id), f'chatllm_project_name': repr(self.chatllm_project_name), f'created_at': repr(
            self.created_at), f'last_event_created_at': repr(self.last_event_created_at), f'conversation_type': repr(self.conversation_type), f'deployment_id': repr(self.deployment_id), f'external_application_id': repr(self.external_application_id)}
        class_name = "ConversationAndProjectSearchResults"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'deployment_conversation_id': self.deployment_conversation_id, 'name': self.name, 'chatllm_project_id': self.chatllm_project_id, 'chatllm_project_name': self.chatllm_project_name, 'created_at': self.created_at,
                'last_event_created_at': self.last_event_created_at, 'conversation_type': self.conversation_type, 'deployment_id': self.deployment_id, 'external_application_id': self.external_application_id}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}



from .return_class import AbstractApiClass


class AudioUrlResult(AbstractApiClass):
    """
        TTS result

        Args:
            client (ApiClient): An authenticated API Client instance
            audioUrl (str): The audio url.
            creditsUsed (float): The credits used.
    """

    def __init__(self, client, audioUrl=None, creditsUsed=None):
        super().__init__(client, None)
        self.audio_url = audioUrl
        self.credits_used = creditsUsed
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'audio_url': repr(
            self.audio_url), f'credits_used': repr(self.credits_used)}
        class_name = "AudioUrlResult"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'audio_url': self.audio_url, 'credits_used': self.credits_used}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}



from .organization_group import OrganizationGroup
from .return_class import AbstractApiClass


class User(AbstractApiClass):
    """
        An Abacus.AI User with a unique user_id.

        Args:
            client (ApiClient): An authenticated API Client instance
            userId (id): The unique identifier of the user.
            name (str): The User's name.
            email (str): The User's primary email address.
            createdAt (str): The date and time when the user joined Abacus.AI.
            status (str): `ACTIVE` when the user has accepted an invite to join the organization, else `INVITED`.
            organizationGroups (OrganizationGroup): List of Organization Groups this user belongs to.
    """

    def __init__(self, client, userId=None, name=None, email=None, createdAt=None, status=None, organizationGroups={}):
        super().__init__(client, userId)
        self.user_id = userId
        self.name = name
        self.email = email
        self.created_at = createdAt
        self.status = status
        self.organization_groups = client._build_class(
            OrganizationGroup, organizationGroups)
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'user_id': repr(self.user_id), f'name': repr(self.name), f'email': repr(self.email), f'created_at': repr(
            self.created_at), f'status': repr(self.status), f'organization_groups': repr(self.organization_groups)}
        class_name = "User"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'user_id': self.user_id, 'name': self.name, 'email': self.email, 'created_at': self.created_at,
                'status': self.status, 'organization_groups': self._get_attribute_as_dict(self.organization_groups)}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}



from .api_class import WorkflowGraphNode
from .return_class import AbstractApiClass


class TemplateNodeDetails(AbstractApiClass):
    """
        Details about WorkflowGraphNode object and notebook code for adding template nodes in workflow.

        Args:
            client (ApiClient): An authenticated API Client instance
            notebookCode (list): The boilerplate code that needs to be shown in notebook for creating workflow graph node using corresponding template.
            workflowGraphNode (WorkflowGraphNode): The workflow graph node object corresponding to the template.
    """

    def __init__(self, client, notebookCode=None, workflowGraphNode={}):
        super().__init__(client, None)
        self.notebook_code = notebookCode
        self.workflow_graph_node = client._build_class(
            WorkflowGraphNode, workflowGraphNode)
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'notebook_code': repr(
            self.notebook_code), f'workflow_graph_node': repr(self.workflow_graph_node)}
        class_name = "TemplateNodeDetails"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'notebook_code': self.notebook_code,
                'workflow_graph_node': self._get_attribute_as_dict(self.workflow_graph_node)}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}



from .return_class import AbstractApiClass


class StreamingRowCount(AbstractApiClass):
    """
        Returns the number of rows in a streaming feature group from the specified time

        Args:
            client (ApiClient): An authenticated API Client instance
            count (int): The number of rows in the feature group
            startTsMs (int): The start time for the number of rows.
    """

    def __init__(self, client, count=None, startTsMs=None):
        super().__init__(client, None)
        self.count = count
        self.start_ts_ms = startTsMs
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'count': repr(self.count),
                     f'start_ts_ms': repr(self.start_ts_ms)}
        class_name = "StreamingRowCount"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'count': self.count, 'start_ts_ms': self.start_ts_ms}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}



from .return_class import AbstractApiClass


class FeatureGroupRowProcessSummary(AbstractApiClass):
    """
        A summary of the feature group processes for a deployment.

        Args:
            client (ApiClient): An authenticated API Client instance
            totalProcesses (int): The total number of processes
            pendingProcesses (int): The number of pending processes
            processingProcesses (int): The number of processes currently processing
            completeProcesses (int): The number of complete processes
            failedProcesses (int): The number of failed processes
    """

    def __init__(self, client, totalProcesses=None, pendingProcesses=None, processingProcesses=None, completeProcesses=None, failedProcesses=None):
        super().__init__(client, None)
        self.total_processes = totalProcesses
        self.pending_processes = pendingProcesses
        self.processing_processes = processingProcesses
        self.complete_processes = completeProcesses
        self.failed_processes = failedProcesses
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'total_processes': repr(self.total_processes), f'pending_processes': repr(self.pending_processes), f'processing_processes': repr(
            self.processing_processes), f'complete_processes': repr(self.complete_processes), f'failed_processes': repr(self.failed_processes)}
        class_name = "FeatureGroupRowProcessSummary"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'total_processes': self.total_processes, 'pending_processes': self.pending_processes, 'processing_processes':
                self.processing_processes, 'complete_processes': self.complete_processes, 'failed_processes': self.failed_processes}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}



from enum import Enum
from typing import Union


def deprecated_enums(*enum_values):
    def enum_class_wrapper(cls):
        cls.__deprecated_values__ = list(enum_values)
        return cls
    return enum_class_wrapper


class ApiEnum(Enum):
    __deprecated_values__ = []

    def is_deprecated(self):
        return self.value in self.__deprecated_values__

    def __eq__(self, other):
        if isinstance(other, str) and isinstance(self.value, str):
            return self.value.upper() == other.upper()
        elif isinstance(other, int):
            return self.value == other
        elif other is None:
            return self.value == ''
        return super().__eq__(other)

    def __hash__(self):
        if isinstance(self.value, str):
            return hash(self.value.upper())
        return hash(self.value)


class ProblemType(ApiEnum):
    AI_AGENT = 'ai_agent'
    EVENT_ANOMALY = 'event_anomaly'
    CLUSTERING = 'clustering'
    CLUSTERING_TIMESERIES = 'clustering_timeseries'
    CUMULATIVE_FORECASTING = 'cumulative_forecasting'
    NAMED_ENTITY_EXTRACTION = 'nlp_ner'
    NATURAL_LANGUAGE_SEARCH = 'nlp_search'
    CHAT_LLM = 'chat_llm'
    SENTENCE_BOUNDARY_DETECTION = 'nlp_sentence_boundary_detection'
    SENTIMENT_DETECTION = 'nlp_sentiment'
    DOCUMENT_CLASSIFICATION = 'nlp_classification'
    DOCUMENT_SUMMARIZATION = 'nlp_summarization'
    DOCUMENT_VISUALIZATION = 'nlp_document_visualization'
    PERSONALIZATION = 'personalization'
    PREDICTIVE_MODELING = 'regression'
    FINETUNED_LLM = 'finetuned_llm'
    FORECASTING = 'forecasting'
    CUSTOM_TRAINED_MODEL = 'plug_and_play'
    CUSTOM_ALGORITHM = 'trainable_plug_and_play'
    FEATURE_STORE = 'feature_store'
    IMAGE_CLASSIFICATION = 'vision_classification'
    OBJECT_DETECTION = 'vision_object_detection'
    IMAGE_VALUE_PREDICTION = 'vision_regression'
    MODEL_MONITORING = 'model_monitoring'
    LANGUAGE_DETECTION = 'language_detection'
    OPTIMIZATION = 'optimization'
    PRETRAINED_MODELS = 'pretrained'
    THEME_ANALYSIS = 'theme_analysis'
    TS_ANOMALY = 'ts_anomaly'


class RegressionObjective(ApiEnum):

    AUC = 'auc'
    ACCURACY = 'acc'
    LOG_LOSS = 'log_loss'
    PRECISION = 'precision'
    RECALL = 'recall'
    F1_SCORE = 'fscore'
    MAE = 'mae'
    MAPE = 'mape'
    WAPE = 'wape'
    RMSE = 'rmse'
    R_SQUARED_COEFFICIENT_OF_DETERMINATION = 'r^2'


class RegressionTreeHPOMode(ApiEnum):
    RAPID = 'rapid'
    THOROUGH = 'thorough'


class PartialDependenceAnalysis(ApiEnum):
    RAPID = 'rapid'
    THOROUGH = 'thorough'


class RegressionAugmentationStrategy(ApiEnum):
    SMOTE = 'smote'
    RESAMPLE = 'resample'


class RegressionTargetTransform(ApiEnum):
    LOG = 'log'
    QUANTILE = 'quantile'
    YEO_JOHNSON = 'yeo-johnson'
    BOX_COX = 'box-cox'


class RegressionTypeOfSplit(ApiEnum):
    RANDOM = 'Random Sampling'
    TIMESTAMP_BASED = 'Timestamp Based'
    ROW_INDICATOR_BASED = 'Row Indicator Based'
    STRATIFIED_RANDOM_SAMPLING = 'Stratified Random Sampling'


class RegressionTimeSplitMethod(ApiEnum):
    TEST_SPLIT_PERCENTAGE_BASED = 'Test Split Percentage Based'
    TEST_START_TIMESTAMP_BASED = 'Test Start Timestamp Based'


class RegressionLossFunction(ApiEnum):
    HUBER = 'Huber'
    MSE = 'Mean Squared Error'
    MAE = 'Mean Absolute Error'
    MAPE = 'Mean Absolute Percentage Error'
    MSLE = 'Mean Squared Logarithmic Error'
    TWEEDIE = 'Tweedie'
    CROSS_ENTROPY = 'Cross Entropy'
    FOCAL_CROSS_ENTROPY = 'Focal Cross Entropy'
    AUTOMATIC = 'Automatic'
    CUSTOM = 'Custom'


class ExplainerType(ApiEnum):
    KERNEL_EXPLAINER = 'KERNEL_EXPLAINER'
    LIME_EXPLAINER = 'LIME_EXPLAINER'
    TREE_EXPLAINER = 'TREE_EXPLAINER'
    EBM_EXPLAINER = 'EBM_EXPLAINER'


class SamplingMethodType(ApiEnum):
    N_SAMPLING = 'N_SAMPLING'
    PERCENT_SAMPLING = 'PERCENT_SAMPLING'


class MergeMode(ApiEnum):
    LAST_N = 'LAST_N'
    TIME_WINDOW = 'TIME_WINDOW'


class OperatorType(ApiEnum):
    UNPIVOT = 'UNPIVOT'
    MARKDOWN = 'MARKDOWN'
    CRAWLER = 'CRAWLER'
    EXTRACT_DOCUMENT_DATA = 'EXTRACT_DOCUMENT_DATA'
    DATA_GENERATION = 'DATA_GENERATION'
    UNION = 'UNION'


class MarkdownOperatorInputType(ApiEnum):
    HTML = 'HTML'


class FillLogic(ApiEnum):
    # back / future
    AVERAGE = 'average'
    MAX = 'max'
    MEDIAN = 'median'
    MIN = 'min'
    CUSTOM = 'custom'
    # middle
    BACKFILL = 'bfill'
    FORWARDFILL = 'ffill'
    LINEAR = 'linear'
    NEAREST = 'nearest'


class BatchSize(ApiEnum):
    BATCH_8 = 8
    BATCH_16 = 16
    BATCH_32 = 32
    BATCH_64 = 64
    BATCH_128 = 128
    BATCH_256 = 256
    BATCH_384 = 384
    BATCH_512 = 512
    BATCH_740 = 740
    BATCH_1024 = 1024


class HolidayCalendars(ApiEnum):
    AU = 'AU'
    UK = 'UK'
    US = 'US'


class FileFormat(ApiEnum):
    AVRO = 'AVRO'
    PARQUET = 'PARQUET'
    TFRECORD = 'TFRECORD'
    TSV = 'TSV'
    CSV = 'CSV'
    ORC = 'ORC'
    JSON = 'JSON'
    ODS = 'ODS'
    XLS = 'XLS'
    GZ = 'GZ'
    ZIP = 'ZIP'
    TAR = 'TAR'
    DOCX = 'DOCX'
    PDF = 'PDF'
    MD = 'md'
    RAR = 'RAR'
    GIF = 'GIF'
    JPEG = 'JPG'
    PNG = 'PNG'
    TIF = 'TIFF'
    NUMBERS = 'NUMBERS'
    PPTX = 'PPTX'
    PPT = 'PPT'
    HTML = 'HTML'
    TXT = 'txt'
    EML = 'eml'
    MP3 = 'MP3'
    MP4 = 'MP4'
    FLV = 'flv'
    MOV = 'mov'
    MPG = 'mpg'
    MPEG = 'mpeg'
    WEBP = 'webp'
    WEBM = 'webm'
    WMV = 'wmv'
    MSG = 'msg'
    TS = 'TS'
    TSX = 'TSX'


class ExperimentationMode(ApiEnum):
    RAPID = 'rapid'
    THOROUGH = 'thorough'


class PersonalizationTrainingMode(ApiEnum):
    EXPERIMENTAL = 'EXP'
    PRODUCTION = 'PROD'


class PersonalizationObjective(ApiEnum):
    NDCG = 'ndcg'
    NDCG_5 = 'ndcg@5'
    NDCG_10 = 'ndcg@10'
    MAP = 'map'
    MAP_5 = 'map@5'
    MAP_10 = 'map@10'
    MRR = 'mrr'
    PERSONALIZATION = 'personalization@10'
    COVERAGE = 'coverage'


# Forecasting
class ForecastingObjective(ApiEnum):
    ACCURACY = 'w_c_accuracy'
    WAPE = 'wape'
    MAPE = 'mape'
    CMAPE = 'cmape'
    RMSE = 'rmse'
    CV = 'coefficient_of_variation'
    BIAS = 'bias'
    SRMSE = 'srmse'


class ForecastingFrequency(ApiEnum):
    HOURLY = '1H'
    DAILY = '1D'
    WEEKLY_SUNDAY_START = '1W'
    WEEKLY_MONDAY_START = 'W-MON'
    WEEKLY_SATURDAY_START = 'W-SAT'
    MONTH_START = 'MS'
    MONTH_END = '1M'
    QUARTER_START = 'QS'
    QUARTER_END = '1Q'
    YEARLY = '1Y'


class ForecastingDataSplitType(ApiEnum):
    AUTO = 'Automatic Time Based'
    TIMESTAMP = 'Timestamp Based'
    ITEM = 'Item Based'
    PREDICTION_LENGTH = 'Force Prediction Length'
    L_SHAPED_AUTO = 'L-shaped Split - Automatic Time Based'
    L_SHAPED_TIMESTAMP = 'L-shaped Split - Timestamp Based'


class ForecastingLossFunction(ApiEnum):
    CUSTOM = 'Custom'
    MEAN_ABSOLUTE_ERROR = 'mae'
    NORMALIZED_MEAN_ABSOLUTE_ERROR = 'nmae'
    PEAKS_MEAN_ABSOLUTE_ERROR = 'peaks_mae'
    MEAN_ABSOLUTE_PERCENTAGE_ERROR = 'stable_mape'
    POINTWISE_ACCURACY = 'accuracy'
    ROOT_MEAN_SQUARE_ERROR = 'rmse'
    NORMALIZED_ROOT_MEAN_SQUARE_ERROR = 'nrmse'
    ASYMMETRIC_MEAN_ABSOLUTE_PERCENTAGE_ERROR = 'asymmetric_mape'
    STABLE_STANDARDIZED_MEAN_ABSOLUTE_PERCENTAGE_ERROR = 'stable_standardized_mape_with_cmape'
    GAUSSIAN = 'mle_gaussian_local'
    GAUSSIAN_FULL_COVARIANCE = 'mle_gaussfullcov'
    GUASSIAN_EXPONENTIAL = 'mle_gaussexp'
    MIX_GAUSSIANS = 'mle_gaussmix'
    WEIBULL = 'mle_weibull'
    NEGATIVE_BINOMIAL = 'mle_negbinom'
    LOG_ROOT_MEAN_SQUARE_ERROR = 'log_rmse'


class ForecastingLocalScaling(ApiEnum):
    ZSCORE = 'zscore'
    SLIDING_ZSCORE = 'sliding_zscore'
    LAST_POINT = 'lastpoint'
    MIN_MAX = 'minmax'
    MIN_STD = 'minstd'
    ROBUST = 'robust'
    ITEM = 'item'


class ForecastingFillMethod(ApiEnum):
    BACK = 'BACK'
    MIDDLE = 'MIDDLE'
    FUTURE = 'FUTURE'


class ForecastingQuanitlesExtensionMethod(ApiEnum):
    DIRECT = 'direct'
    QUADRATIC = 'quadratic'
    ANCESTRAL_SIMULATION = 'simulation'


# Timeseries Anomaly Detection
class TimeseriesAnomalyDataSplitType(ApiEnum):
    AUTO = 'Automatic Time Based'
    TIMESTAMP = 'Fixed Timestamp Based'


class TimeseriesAnomalyTypeOfAnomaly(ApiEnum):
    HIGH_PEAK = 'high_peak'
    LOW_PEAK = 'low_peak'


class TimeseriesAnomalyUseHeuristic(ApiEnum):
    ENABLE = 'enable'
    DISABLE = 'disable'
    AUTOMATIC = 'automatic'


# Named Entity Recognition
class NERObjective(ApiEnum):
    LOG_LOSS = 'log_loss'
    AUC = 'auc'
    PRECISION = 'precision'
    RECALL = 'recall'
    ANNOTATIONS_PRECISION = 'annotations_precision'
    ANNOTATIONS_RECALL = 'annotations_recall'


# Named Entity Recognition


class NERModelType(ApiEnum):
    PRETRAINED_BERT = 'pretrained_bert'
    PRETRAINED_ROBERTA_27 = 'pretrained_roberta_27'
    PRETRAINED_ROBERTA_43 = 'pretrained_roberta_43'
    PRETRAINED_MULTILINGUAL = 'pretrained_multilingual'
    LEARNED = 'learned'


class NLPDocumentFormat(ApiEnum):
    AUTO = 'auto'
    TEXT = 'text'
    DOC = 'doc'
    TOKENS = 'tokens'


# Sentiment Analysis
class SentimentType(ApiEnum):
    VALENCE = 'valence'
    EMOTION = 'emotion'


# Timeseries Clustering
class ClusteringImputationMethod(ApiEnum):
    AUTOMATIC = 'Automatic'
    ZEROS = 'Zeros'
    INTERPOLATE = 'Interpolate'


class ConnectorType(ApiEnum):
    FILE = 'FILE'
    DATABASE = 'DATABASE'
    STREAMING = 'STREAMING'
    APPLICATION = 'APPLICATION'


class ApplicationConnectorType(ApiEnum):
    GOOGLEANALYTICS = 'GOOGLEANALYTICS'
    GOOGLEDRIVE = 'GOOGLEDRIVE'
    GOOGLECALENDAR = 'GOOGLECALENDAR'
    GIT = 'GIT'
    CONFLUENCE = 'CONFLUENCE'
    JIRA = 'JIRA'
    ONEDRIVE = 'ONEDRIVE'
    ZENDESK = 'ZENDESK'
    SLACK = 'SLACK'
    SHAREPOINT = 'SHAREPOINT'
    TEAMS = 'TEAMS'
    ABACUSUSAGEMETRICS = 'ABACUSUSAGEMETRICS'
    MICROSOFTAUTH = 'MICROSOFTAUTH'
    FRESHSERVICE = 'FRESHSERVICE'
    ZENDESKSUNSHINEMESSAGING = 'ZENDESKSUNSHINEMESSAGING'
    GOOGLEDRIVEUSER = 'GOOGLEDRIVEUSER'
    GOOGLEWORKSPACEUSER = 'GOOGLEWORKSPACEUSER'
    GMAILUSER = 'GMAILUSER'
    GOOGLESHEETS = 'GOOGLESHEETS'
    GOOGLEDOCS = 'GOOGLEDOCS'
    TEAMSSCRAPER = 'TEAMSSCRAPER'
    GITHUBUSER = 'GITHUBUSER'
    OKTASAML = 'OKTASAML'
    BOX = 'BOX'
    SFTPAPPLICATION = 'SFTPAPPLICATION'
    OAUTH = 'OAUTH'
    SALESFORCE = 'SALESFORCE'
    TWITTER = 'TWITTER'
    MCP = 'MCP'
    ODBC = 'ODBC'
    DBC = 'DBC'
    GENERIC_OAUTH = 'GENERIC_OAUTH'
    OUTLOOK = 'OUTLOOK'
    BIGQUERY = 'BIGQUERY'
    AZURESTORAGE = 'AZURESTORAGE'
    SHOPIFY = 'SHOPIFY'

    @classmethod
    def user_connectors(cls):
        return [
            cls.GOOGLEDRIVEUSER,
            cls.GOOGLECALENDAR,
            cls.GMAILUSER,
            cls.SLACK,
            cls.JIRA,
            cls.ONEDRIVE,
            cls.SALESFORCE,
            cls.TWITTER,
            cls.SHAREPOINT,
            cls.TEAMS,
            cls.CONFLUENCE,
            cls.BOX,
            cls.GITHUBUSER,
            cls.ODBC,
            cls.DBC,
            cls.GENERIC_OAUTH,
            cls.OUTLOOK,
            cls.BIGQUERY,
            cls.SHOPIFY
        ]

    @classmethod
    def database_connectors(cls):
        return [cls.SALESFORCE, cls.ODBC, cls.DBC, cls.BIGQUERY]


class StreamingConnectorType(ApiEnum):
    KAFKA = 'KAFKA'


class PythonFunctionArgumentType(ApiEnum):
    FEATURE_GROUP = 'FEATURE_GROUP'
    INTEGER = 'INTEGER'
    STRING = 'STRING'
    BOOLEAN = 'BOOLEAN'
    FLOAT = 'FLOAT'
    JSON = 'JSON'
    LIST = 'LIST'
    DATASET_ID = 'DATASET_ID'
    MODEL_ID = 'MODEL_ID'
    FEATURE_GROUP_ID = 'FEATURE_GROUP_ID'
    MONITOR_ID = 'MONITOR_ID'
    BATCH_PREDICTION_ID = 'BATCH_PREDICTION_ID'
    DEPLOYMENT_ID = 'DEPLOYMENT_ID'
    ATTACHMENT = 'ATTACHMENT'

    @staticmethod
    def to_json_type(type):
        if type == PythonFunctionArgumentType.INTEGER or type == PythonFunctionArgumentType.FLOAT:
            return 'number'
        elif type == PythonFunctionArgumentType.STRING or type == PythonFunctionArgumentType.ATTACHMENT:
            return 'string'
        elif type == PythonFunctionArgumentType.BOOLEAN:
            return 'boolean'
        elif type == PythonFunctionArgumentType.LIST:
            return 'array'
        elif type == PythonFunctionArgumentType.JSON:
            return 'object'
        else:
            raise ValueError(f'Invalid type: {type}. type not JSON compatible')


class PythonFunctionOutputArgumentType(ApiEnum):
    NTEGER = 'INTEGER'
    STRING = 'STRING'
    BOOLEAN = 'BOOLEAN'
    FLOAT = 'FLOAT'
    JSON = 'JSON'
    LIST = 'LIST'
    DATASET_ID = 'DATASET_ID'
    MODEL_ID = 'MODEL_ID'
    FEATURE_GROUP_ID = 'FEATURE_GROUP_ID'
    MONITOR_ID = 'MONITOR_ID'
    BATCH_PREDICTION_ID = 'BATCH_PREDICTION_ID'
    DEPLOYMENT_ID = 'DEPLOYMENT_ID'
    ANY = 'ANY'
    ATTACHMENT = 'ATTACHMENT'


class VectorStoreTextEncoder(ApiEnum):
    E5 = 'E5'
    OPENAI = 'OPENAI'
    OPENAI_COMPACT = 'OPENAI_COMPACT'
    OPENAI_LARGE = 'OPENAI_LARGE'
    SENTENCE_BERT = 'SENTENCE_BERT'
    E5_SMALL = 'E5_SMALL'
    CODE_BERT = 'CODE_BERT'


@deprecated_enums('OPENAI_GPT4_32K', 'OPENAI_GPT3_5', 'OPENAI_GPT3_5_TEXT',
                  'OPENAI_GPT4', 'OPENAI_GPT4_128K', 'OPENAI_GPT4_128K_LATEST',
                  'OPENAI_O1_MINI',
                  'GEMINI_1_5_PRO', 'GEMINI_1_5_FLASH', 'GEMINI_2_PRO', 'GEMINI_2_FLASH_THINKING',
                  'XAI_GROK',
                  'CLAUDE_V3_OPUS', 'CLAUDE_V3_HAIKU',
                  'LLAMA3_LARGE_CHAT', 'QWEN_2_5_32B_BASE')
class LLMName(ApiEnum):
    OPENAI_GPT3_5 = 'OPENAI_GPT3_5'
    OPENAI_GPT3_5_TEXT = 'OPENAI_GPT3_5_TEXT'
    OPENAI_GPT4 = 'OPENAI_GPT4'
    OPENAI_GPT4_32K = 'OPENAI_GPT4_32K'
    OPENAI_GPT4_128K = 'OPENAI_GPT4_128K'
    OPENAI_GPT4_128K_LATEST = 'OPENAI_GPT4_128K_LATEST'
    OPENAI_GPT4O = 'OPENAI_GPT4O'
    OPENAI_GPT4O_MINI = 'OPENAI_GPT4O_MINI'
    OPENAI_O1_MINI = 'OPENAI_O1_MINI'
    OPENAI_O3 = 'OPENAI_O3'
    OPENAI_O3_HIGH = 'OPENAI_O3_HIGH'
    OPENAI_O4_MINI = 'OPENAI_O4_MINI'
    OPENAI_O4_MINI_HIGH = 'OPENAI_O4_MINI_HIGH'
    OPENAI_GPT4_1 = 'OPENAI_GPT4_1'
    OPENAI_GPT4_1_MINI = 'OPENAI_GPT4_1_MINI'
    OPENAI_GPT4_1_NANO = 'OPENAI_GPT4_1_NANO'
    OPENAI_GPT5 = 'OPENAI_GPT5'
    OPENAI_GPT5_MINI = 'OPENAI_GPT5_MINI'
    OPENAI_GPT5_MINI_HIGH = 'OPENAI_GPT5_MINI_HIGH'
    OPENAI_GPT5_MINI_LOW = 'OPENAI_GPT5_MINI_LOW'
    OPENAI_GPT5_NANO = 'OPENAI_GPT5_NANO'
    OPENAI_GPT5_NANO_HIGH = 'OPENAI_GPT5_NANO_HIGH'
    OPENAI_GPT5_1 = 'OPENAI_GPT5_1'
    OPENAI_GPT5_2 = 'OPENAI_GPT5_2'
    CLAUDE_V3_HAIKU = 'CLAUDE_V3_HAIKU'
    CLAUDE_V3_OPUS = 'CLAUDE_V3_OPUS'
    CLAUDE_V3_5_HAIKU = 'CLAUDE_V3_5_HAIKU'
    CLAUDE_V3_5_SONNET = 'CLAUDE_V3_5_SONNET'
    CLAUDE_V3_7_SONNET = 'CLAUDE_V3_7_SONNET'
    CLAUDE_V4_SONNET = 'CLAUDE_V4_SONNET'
    CLAUDE_V4_OPUS = 'CLAUDE_V4_OPUS'
    CLAUDE_V4_5_SONNET = 'CLAUDE_V4_5_SONNET'
    GEMINI_1_5_PRO = 'GEMINI_1_5_PRO'
    GEMINI_1_5_FLASH = 'GEMINI_1_5_FLASH'
    GEMINI_2_PRO = 'GEMINI_2_PRO'
    GEMINI_2_FLASH = 'GEMINI_2_FLASH'
    GEMINI_2_5_PRO = 'GEMINI_2_5_PRO'
    GEMINI_3_PRO = 'GEMINI_3_PRO'
    GEMINI_2_5_FLASH = 'GEMINI_2_5_FLASH'
    GEMINI_3_FLASH = 'GEMINI_3_FLASH'
    XAI_GROK = 'XAI_GROK'
    XAI_GROK_3 = 'XAI_GROK_3'
    XAI_GROK_3_MINI = 'XAI_GROK_3_MINI'
    XAI_GROK_4 = 'XAI_GROK_4'
    LLAMA4_MAVERICK = 'LLAMA4_MAVERICK'
    LLAMA3_1_405B = 'LLAMA3_1_405B'
    LLAMA3_1_70B = 'LLAMA3_1_70B'
    LLAMA3_1_8B = 'LLAMA3_1_8B'
    LLAMA3_3_70B = 'LLAMA3_3_70B'
    LLAMA3_LARGE_CHAT = 'LLAMA3_LARGE_CHAT'
    ABACUS_SMAUG3 = 'ABACUS_SMAUG3'
    ABACUS_DRACARYS = 'ABACUS_DRACARYS'
    QWEN_2_5_32B = 'QWEN_2_5_32B'
    QWEN_2_5_32B_BASE = 'QWEN_2_5_32B_BASE'
    QWEN_2_5_72B = 'QWEN_2_5_72B'
    QWQ_32B = 'QWQ_32B'
    QWEN3_32B = 'QWEN3_32B'
    QWEN3_235B_A22B = 'QWEN3_235B_A22B'
    QWEN3_CODER = 'QWEN3_CODER'
    DEEPSEEK_V3_1 = 'DEEPSEEK_V3_1'
    DEEPSEEK_R1 = 'DEEPSEEK_R1'


class MonitorAlertType(ApiEnum):
    ACCURACY_BELOW_THRESHOLD = 'AccuracyBelowThreshold'
    FEATURE_DRIFT = 'FeatureDrift'
    DATA_INTEGRITY_VIOLATIONS = 'DataIntegrityViolations'
    BIAS_VIOLATIONS = 'BiasViolations'
    HISTORY_LENGTH_DRIFT = 'HistoryLengthDrift'
    TARGET_DRIFT = 'TargetDrift'
    PREDICTION_COUNT = 'PredictionCount'


class FeatureDriftType(ApiEnum):
    KL = 'kl'
    KS = 'ks'
    WS = 'ws'
    JS = 'js'
    PSI = 'psi'
    CHI_SQUARE = 'chi_square'
    CSI = 'csi'


class DataIntegrityViolationType(ApiEnum):
    NULL_VIOLATIONS = 'null_violations'
    RANGE_VIOLATIONS = 'range_violations'
    CATEGORICAL_RANGE_VIOLATION = 'categorical_range_violations'
    TOTAL_VIOLATIONS = 'total_violations'


class BiasType(ApiEnum):
    DEMOGRAPHIC_PARITY = 'demographic_parity'
    EQUAL_OPPORTUNITY = 'equal_opportunity'
    GROUP_BENEFIT_EQUALITY = 'group_benefit'
    TOTAL = 'total'


class AlertActionType(ApiEnum):
    EMAIL = 'Email'


class PythonFunctionType(ApiEnum):
    FEATURE_GROUP = 'FEATURE_GROUP'
    PLOTLY_FIG = 'PLOTLY_FIG'
    STEP_FUNCTION = 'STEP_FUNCTION'
    USERCODE_TOOL = 'USERCODE_TOOL'
    CONNECTOR_TOOL = 'CONNECTOR_TOOL'


class EvalArtifactType(ApiEnum):
    FORECASTING_ACCURACY = 'bar_chart'
    FORECASTING_VOLUME = 'bar_chart_volume'
    FORECASTING_HISTORY_LENGTH_ACCURACY = 'bar_chart_accuracy_by_history'


class FieldDescriptorType(ApiEnum):
    STRING = 'STRING'
    INTEGER = 'INTEGER'
    FLOAT = 'FLOAT'
    BOOLEAN = 'BOOLEAN'
    DATETIME = 'DATETIME'
    DATE = 'DATE'


class WorkflowNodeInputType(ApiEnum):
    # Duplicated in reainternal.enums, both should be kept in sync
    USER_INPUT = 'USER_INPUT'
    WORKFLOW_VARIABLE = 'WORKFLOW_VARIABLE'
    IGNORE = 'IGNORE'
    CONSTANT = 'CONSTANT'


class WorkflowNodeOutputType(ApiEnum):
    ATTACHMENT = 'ATTACHMENT'
    BOOLEAN = 'BOOLEAN'
    FLOAT = 'FLOAT'
    INTEGER = 'INTEGER'
    DICT = 'DICT'
    LIST = 'LIST'
    STRING = 'STRING'
    RUNTIME_SCHEMA = 'RUNTIME_SCHEMA'
    ANY = 'ANY'
    SIGNAL = 'SIGNAL'  # Control flow only - triggers downstream execution without passing data

    @classmethod
    def normalize_type(cls, python_type: Union[str, type, None, 'WorkflowNodeOutputType', 'PythonFunctionOutputArgumentType']) -> 'WorkflowNodeOutputType':
        if isinstance(python_type, WorkflowNodeOutputType):
            return python_type

        if isinstance(python_type, PythonFunctionOutputArgumentType):
            if python_type.value in cls.__members__:
                return cls(python_type.value)
            else:
                return cls.ANY

        if isinstance(python_type, type) or isinstance(python_type, str):
            python_type = python_type.__name__ if isinstance(python_type, type) else python_type
        else:
            python_type = type(python_type).__name__

        if python_type == 'int':
            return cls.INTEGER
        elif python_type == 'float':
            return cls.FLOAT
        elif python_type == 'str':
            return cls.STRING
        elif python_type == 'bool':
            return cls.BOOLEAN
        elif python_type == 'dict':
            return cls.DICT
        elif python_type == 'list':
            return cls.LIST
        elif python_type in ('NoneType', '_SpecialForm'):
            return cls.ANY
        raise ValueError(f'Unsupported output type: {python_type}')


class OcrMode(ApiEnum):
    AUTO = 'AUTO'
    DEFAULT = 'DEFAULT'
    LAYOUT = 'LAYOUT'
    SCANNED = 'SCANNED'
    COMPREHENSIVE = 'COMPREHENSIVE'
    COMPREHENSIVE_V2 = 'COMPREHENSIVE_V2'
    COMPREHENSIVE_TABLE_MD = 'COMPREHENSIVE_TABLE_MD'
    COMPREHENSIVE_FORM_MD = 'COMPREHENSIVE_FORM_MD'
    COMPREHENSIVE_FORM_AND_TABLE_MD = 'COMPREHENSIVE_FORM_AND_TABLE_MD'
    TESSERACT_FAST = 'TESSERACT_FAST'
    LLM = 'LLM'
    AUGMENTED_LLM = 'AUGMENTED_LLM'

    @classmethod
    def advanced_ocr_modes(cls):
        return [cls.COMPREHENSIVE_V2, cls.COMPREHENSIVE_TABLE_MD, cls.COMPREHENSIVE_FORM_MD, cls.COMPREHENSIVE_FORM_AND_TABLE_MD]


class DocumentType(ApiEnum):
    SIMPLE_TEXT = 'SIMPLE_TEXT'                         # digital text
    TEXT = 'TEXT'                                       # general text with OCR
    TABLES_AND_FORMS = 'TABLES_AND_FORMS'               # tables and forms with OCR
    EMBEDDED_IMAGES = 'EMBEDDED_IMAGES'                 # embedded images with OCR TODO: remove?
    SCANNED_TEXT = 'SCANNED_TEXT'                       # scanned text with OCR
    COMPREHENSIVE_MARKDOWN = 'COMPREHENSIVE_MARKDOWN'   # comprehensive text with Gemini OCR

    @classmethod
    def is_ocr_forced(cls, document_type: 'DocumentType'):
        return document_type in [cls.TEXT, cls.TABLES_AND_FORMS, cls.EMBEDDED_IMAGES, cls.SCANNED_TEXT]


class StdDevThresholdType(ApiEnum):
    ABSOLUTE = 'ABSOLUTE'
    PERCENTILE = 'PERCENTILE'
    STDDEV = 'STDDEV'


class DataType(ApiEnum):
    INTEGER = 'integer'
    FLOAT = 'float'
    STRING = 'string'
    DATE = 'date'
    DATETIME = 'datetime'
    BOOLEAN = 'boolean'
    LIST = 'list'
    STRUCT = 'struct'
    NULL = 'null'
    BINARY = 'binary'


class AgentInterface(ApiEnum):
    # Duplicated in reainternal.enums, both should be kept in sync
    DEFAULT = 'DEFAULT'
    CHAT = 'CHAT'
    MATRIX = 'MATRIX'
    AUTONOMOUS = 'AUTONOMOUS'


class AutonomousTriggerType(ApiEnum):
    """Type of trigger for autonomous agents."""
    SCHEDULE = 'SCHEDULE'
    WEBHOOK = 'WEBHOOK'


class WorkflowNodeTemplateType(ApiEnum):
    TRIGGER = 'trigger'
    DEFAULT = 'default'


class ProjectConfigType(ApiEnum):
    CONSTRAINTS = 'CONSTRAINTS'
    CHAT_FEEDBACK = 'CHAT_FEEDBACK'
    REVIEW_MODE = 'REVIEW_MODE'


class CPUSize(ApiEnum):
    SMALL = 'small'
    MEDIUM = 'medium'
    LARGE = 'large'


class MemorySize(ApiEnum):
    SMALL = 16
    MEDIUM = 32
    LARGE = 64
    XLARGE = 128

    @classmethod
    def from_value(cls, value):
        sorted_members = sorted(cls, key=lambda mem: mem.value)
        for member in sorted_members:
            if member.value >= value:
                return member
        return None


class ResponseSectionType(ApiEnum):
    AGENT_FLOW_BUTTON = 'agent_flow_button'
    ATTACHMENTS = 'attachments'
    BASE64_IMAGE = 'base64_image'
    CHART = 'chart'
    CODE = 'code'
    COLLAPSIBLE_COMPONENT = 'collapsible_component'
    IMAGE_URL = 'image_url'
    RUNTIME_SCHEMA = 'runtime_schema'
    LIST = 'list'
    TABLE = 'table'
    TEXT = 'text'


class CodeLanguage(ApiEnum):
    PYTHON = 'python'
    SQL = 'sql'


class DeploymentConversationType(ApiEnum):
    CHAT_LLM = 'CHATLLM'
    SIMPLE_AGENT = 'SIMPLE_AGENT'
    COMPLEX_AGENT = 'COMPLEX_AGENT'
    WORKFLOW_AGENT = 'WORKFLOW_AGENT'
    COPILOT = 'COPILOT'
    AGENT_CONTROLLER = 'AGENT_CONTROLLER'
    CODE_LLM = 'CODE_LLM'
    CODE_LLM_AGENT = 'CODE_LLM_AGENT'
    CHAT_LLM_TASK = 'CHAT_LLM_TASK'
    COMPUTER_AGENT = 'COMPUTER_AGENT'
    SEARCH_LLM = 'SEARCH_LLM'
    APP_LLM = 'APP_LLM'
    TEST_AGENT = 'TEST_AGENT'
    SUPER_AGENT = 'SUPER_AGENT'
    CODE_LLM_NON_INTERACTIVE = 'CODE_LLM_NON_INTERACTIVE'
    BROWSER_EXTENSION = 'BROWSER_EXTENSION'


class AgentClientType(ApiEnum):
    CHAT_UI = 'CHAT_UI'
    MESSAGING_APP = 'MESSAGING_APP'
    API = 'API'


class OrganizationSecretType(ApiEnum):
    """Enum for organization secret types"""
    ORG_SECRET = 'ORG_SECRET'
    ORG_API_CREDENTIALS = 'ORG_API_CREDENTIALS'
    USER_API_CREDENTIALS = 'USER_API_CREDENTIALS'


class DaemonTaskLifecycleUpdateAction(ApiEnum):
    """Action to perform on daemon task lifecycle"""
    DELETE = 'DELETE'
    PAUSE = 'PAUSE'
    RESUME = 'RESUME'



from .return_class import AbstractApiClass


class LlmArtifact(AbstractApiClass):
    """
        LLM Artifact

        Args:
            client (ApiClient): An authenticated API Client instance
            llmArtifactId (id): The ID of the LLM artifact
            info (dict): The info of the LLM artifact
            description (str): The description of the LLM artifact
            createdAt (str): The creation timestamp
            webAppDeploymentId (id): The ID of the associated web app deployment
            deploymentStatus (str): The status of the associated web app deployment
            isLatest (bool): Whether it is the most recent version of the artifact
            deploymentConversationId (id): The ID of the deployment conversation
            webAppProjectId (id): The ID of the web app project
            hasDatabase (bool): Whether the app associated with the artifact has a database
            hasStorage (bool): Whether the app associated with the artifact has a storage
            projectMetadata (dict): The metadata of the web app project
            hasDatabaseSnapshot (bool): Whether the artifact database has a snapshot
            supportsPreviewRestore (bool): Whether the artifact supports preview restore
            accessLevel (str): The access level of the artifact
    """

    def __init__(self, client, llmArtifactId=None, info=None, description=None, createdAt=None, webAppDeploymentId=None, deploymentStatus=None, isLatest=None, deploymentConversationId=None, webAppProjectId=None, hasDatabase=None, hasStorage=None, projectMetadata=None, hasDatabaseSnapshot=None, supportsPreviewRestore=None, accessLevel=None):
        super().__init__(client, llmArtifactId)
        self.llm_artifact_id = llmArtifactId
        self.info = info
        self.description = description
        self.created_at = createdAt
        self.web_app_deployment_id = webAppDeploymentId
        self.deployment_status = deploymentStatus
        self.is_latest = isLatest
        self.deployment_conversation_id = deploymentConversationId
        self.web_app_project_id = webAppProjectId
        self.has_database = hasDatabase
        self.has_storage = hasStorage
        self.project_metadata = projectMetadata
        self.has_database_snapshot = hasDatabaseSnapshot
        self.supports_preview_restore = supportsPreviewRestore
        self.access_level = accessLevel
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'llm_artifact_id': repr(self.llm_artifact_id), f'info': repr(self.info), f'description': repr(self.description), f'created_at': repr(self.created_at), f'web_app_deployment_id': repr(self.web_app_deployment_id), f'deployment_status': repr(self.deployment_status), f'is_latest': repr(self.is_latest), f'deployment_conversation_id': repr(
            self.deployment_conversation_id), f'web_app_project_id': repr(self.web_app_project_id), f'has_database': repr(self.has_database), f'has_storage': repr(self.has_storage), f'project_metadata': repr(self.project_metadata), f'has_database_snapshot': repr(self.has_database_snapshot), f'supports_preview_restore': repr(self.supports_preview_restore), f'access_level': repr(self.access_level)}
        class_name = "LlmArtifact"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'llm_artifact_id': self.llm_artifact_id, 'info': self.info, 'description': self.description, 'created_at': self.created_at, 'web_app_deployment_id': self.web_app_deployment_id, 'deployment_status': self.deployment_status, 'is_latest': self.is_latest, 'deployment_conversation_id': self.deployment_conversation_id,
                'web_app_project_id': self.web_app_project_id, 'has_database': self.has_database, 'has_storage': self.has_storage, 'project_metadata': self.project_metadata, 'has_database_snapshot': self.has_database_snapshot, 'supports_preview_restore': self.supports_preview_restore, 'access_level': self.access_level}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}



from .email_domain import EmailDomain
from .return_class import AbstractApiClass


class WebAppProject(AbstractApiClass):
    """
        Web app project (_describeWebAppProject).

        Args:
            client (ApiClient): An authenticated API Client instance
            webAppProjectId (id): The project ID
            projectType (str): web_app | web_service | mobile_app
            emailDomains (EmailDomain): List of EmailDomain (custom domains and email status for this project)
    """

    def __init__(self, client, webAppProjectId=None, projectType=None, emailDomains={}):
        super().__init__(client, webAppProjectId)
        self.web_app_project_id = webAppProjectId
        self.project_type = projectType
        self.email_domains = client._build_class(EmailDomain, emailDomains)
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'web_app_project_id': repr(self.web_app_project_id), f'project_type': repr(
            self.project_type), f'email_domains': repr(self.email_domains)}
        class_name = "WebAppProject"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'web_app_project_id': self.web_app_project_id, 'project_type': self.project_type,
                'email_domains': self._get_attribute_as_dict(self.email_domains)}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}



from .return_class import AbstractApiClass


class McpServerConnection(AbstractApiClass):
    """
        Model Context Protocol Server Connection

        Args:
            client (ApiClient): An authenticated API Client instance
            mcpServerConnectionId (id): the id of the MCP server connection.
            createdAt (str): the date and time the MCP server connection was created.
            updatedAt (str): the date and time the MCP server connection was updated.
            name (str): The name of the MCP server.
            config (dict): a dictionary containing the config for the MCP server.
            description (str): description of what the MCP server does.
            transport (str): the transport type for the MCP server.
            authType (str): the auth type for the MCP server.
            externalConnectionId (id): the external connection id for the MCP server.
            inactive (bool): whether the MCP server is inactive.
            tools (list): the tools for the MCP server.
            errorMsg (str): the error message for the MCP server.
            metadata (dict): the metadata for the MCP server.
    """

    def __init__(self, client, mcpServerConnectionId=None, createdAt=None, updatedAt=None, name=None, config=None, description=None, transport=None, authType=None, externalConnectionId=None, inactive=None, tools=None, errorMsg=None, metadata=None):
        super().__init__(client, mcpServerConnectionId)
        self.mcp_server_connection_id = mcpServerConnectionId
        self.created_at = createdAt
        self.updated_at = updatedAt
        self.name = name
        self.config = config
        self.description = description
        self.transport = transport
        self.auth_type = authType
        self.external_connection_id = externalConnectionId
        self.inactive = inactive
        self.tools = tools
        self.error_msg = errorMsg
        self.metadata = metadata
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'mcp_server_connection_id': repr(self.mcp_server_connection_id), f'created_at': repr(self.created_at), f'updated_at': repr(self.updated_at), f'name': repr(self.name), f'config': repr(self.config), f'description': repr(self.description), f'transport': repr(
            self.transport), f'auth_type': repr(self.auth_type), f'external_connection_id': repr(self.external_connection_id), f'inactive': repr(self.inactive), f'tools': repr(self.tools), f'error_msg': repr(self.error_msg), f'metadata': repr(self.metadata)}
        class_name = "McpServerConnection"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'mcp_server_connection_id': self.mcp_server_connection_id, 'created_at': self.created_at, 'updated_at': self.updated_at, 'name': self.name, 'config': self.config, 'description': self.description,
                'transport': self.transport, 'auth_type': self.auth_type, 'external_connection_id': self.external_connection_id, 'inactive': self.inactive, 'tools': self.tools, 'error_msg': self.error_msg, 'metadata': self.metadata}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}



from .return_class import AbstractApiClass


class FsEntry(AbstractApiClass):
    """
        File system entry.

        Args:
            client (ApiClient): An authenticated API Client instance
            name (str): The name of the file/folder
            type (str): The type of entry (file/folder)
            path (str): The path of the entry
            size (int): The size of the entry in bytes
            modified (int): The last modified timestamp
            isFolderEmpty (bool): Whether the folder is empty (only for folders)
            children (list): List of child FSEntry objects (only for folders)
    """

    def __init__(self, client, name=None, type=None, path=None, size=None, modified=None, isFolderEmpty=None, children=None):
        super().__init__(client, None)
        self.name = name
        self.type = type
        self.path = path
        self.size = size
        self.modified = modified
        self.isFolderEmpty = isFolderEmpty
        self.children = children
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'name': repr(self.name), f'type': repr(self.type), f'path': repr(self.path), f'size': repr(
            self.size), f'modified': repr(self.modified), f'isFolderEmpty': repr(self.isFolderEmpty), f'children': repr(self.children)}
        class_name = "FsEntry"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'name': self.name, 'type': self.type, 'path': self.path, 'size': self.size,
                'modified': self.modified, 'isFolderEmpty': self.isFolderEmpty, 'children': self.children}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}



from .return_class import AbstractApiClass


class ToneDetails(AbstractApiClass):
    """
        Tone details for audio

        Args:
            client (ApiClient): An authenticated API Client instance
            voiceId (str): The voice id
            name (str): The name
            gender (str): The gender
            language (str): The language
            age (str): The age
            accent (str): The accent
            useCase (str): The use case
            description (str): The description
    """

    def __init__(self, client, voiceId=None, name=None, gender=None, language=None, age=None, accent=None, useCase=None, description=None):
        super().__init__(client, None)
        self.voice_id = voiceId
        self.name = name
        self.gender = gender
        self.language = language
        self.age = age
        self.accent = accent
        self.use_case = useCase
        self.description = description
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'voice_id': repr(self.voice_id), f'name': repr(self.name), f'gender': repr(self.gender), f'language': repr(
            self.language), f'age': repr(self.age), f'accent': repr(self.accent), f'use_case': repr(self.use_case), f'description': repr(self.description)}
        class_name = "ToneDetails"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'voice_id': self.voice_id, 'name': self.name, 'gender': self.gender, 'language': self.language,
                'age': self.age, 'accent': self.accent, 'use_case': self.use_case, 'description': self.description}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}



from .return_class import AbstractApiClass


class CustomTrainFunctionInfo(AbstractApiClass):
    """
        Information about how to call the customer provided train function.

        Args:
            client (ApiClient): An authenticated API Client instance
            trainingDataParameterNameMapping (dict): The mapping from feature group type to the dataframe parameter name
            schemaMappings (dict): The feature type to feature name mapping for each dataframe
            trainDataParameterToFeatureGroupIds (dict): The mapping from the dataframe parameter name to the feature group id backing the data
            trainingConfig (dict): The configs for training
    """

    def __init__(self, client, trainingDataParameterNameMapping=None, schemaMappings=None, trainDataParameterToFeatureGroupIds=None, trainingConfig=None):
        super().__init__(client, None)
        self.training_data_parameter_name_mapping = trainingDataParameterNameMapping
        self.schema_mappings = schemaMappings
        self.train_data_parameter_to_feature_group_ids = trainDataParameterToFeatureGroupIds
        self.training_config = trainingConfig
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'training_data_parameter_name_mapping': repr(self.training_data_parameter_name_mapping), f'schema_mappings': repr(
            self.schema_mappings), f'train_data_parameter_to_feature_group_ids': repr(self.train_data_parameter_to_feature_group_ids), f'training_config': repr(self.training_config)}
        class_name = "CustomTrainFunctionInfo"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'training_data_parameter_name_mapping': self.training_data_parameter_name_mapping, 'schema_mappings': self.schema_mappings,
                'train_data_parameter_to_feature_group_ids': self.train_data_parameter_to_feature_group_ids, 'training_config': self.training_config}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}



from .return_class import AbstractApiClass


class RazorpayPaymentRequirements(AbstractApiClass):
    """
        A Razorpay customer and order

        Args:
            client (ApiClient): An authenticated API Client instance
            customerId (str): The unique identifier of the customer.
            orderId (str): The unique identifier of the order.
            invoiceId (str): The unique identifier of the invoice.
    """

    def __init__(self, client, customerId=None, orderId=None, invoiceId=None):
        super().__init__(client, None)
        self.customer_id = customerId
        self.order_id = orderId
        self.invoice_id = invoiceId
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'customer_id': repr(self.customer_id), f'order_id': repr(
            self.order_id), f'invoice_id': repr(self.invoice_id)}
        class_name = "RazorpayPaymentRequirements"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'customer_id': self.customer_id,
                'order_id': self.order_id, 'invoice_id': self.invoice_id}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}



from .return_class import AbstractApiClass


class SttGenModelOptions(AbstractApiClass):
    """
        STT generation model options

        Args:
            client (ApiClient): An authenticated API Client instance
            keys (list): The keys of the image generation model options represented as the enum values.
            values (list): The display names of the image generation model options.
    """

    def __init__(self, client, keys=None, values=None):
        super().__init__(client, None)
        self.keys = keys
        self.values = values
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'keys': repr(self.keys), f'values': repr(self.values)}
        class_name = "SttGenModelOptions"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'keys': self.keys, 'values': self.values}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}



from .custom_metric_version import CustomMetricVersion
from .return_class import AbstractApiClass


class CustomMetric(AbstractApiClass):
    """
        Custom metric.

        Args:
            client (ApiClient): An authenticated API Client instance
            customMetricId (str): Unique string identifier of the custom metric.
            name (str): Name assigned to the custom metric.
            createdAt (str): Date and time when the custom metric was created (ISO 8601 format).
            problemType (str): Problem type that this custom metric is applicable to (e.g. regression).
            notebookId (str): Unique string identifier of the notebook used to create/edit the custom metric.
            latestCustomMetricVersion (CustomMetricVersion): Latest version of the custom metric.
    """

    def __init__(self, client, customMetricId=None, name=None, createdAt=None, problemType=None, notebookId=None, latestCustomMetricVersion={}):
        super().__init__(client, customMetricId)
        self.custom_metric_id = customMetricId
        self.name = name
        self.created_at = createdAt
        self.problem_type = problemType
        self.notebook_id = notebookId
        self.latest_custom_metric_version = client._build_class(
            CustomMetricVersion, latestCustomMetricVersion)
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'custom_metric_id': repr(self.custom_metric_id), f'name': repr(self.name), f'created_at': repr(self.created_at), f'problem_type': repr(
            self.problem_type), f'notebook_id': repr(self.notebook_id), f'latest_custom_metric_version': repr(self.latest_custom_metric_version)}
        class_name = "CustomMetric"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'custom_metric_id': self.custom_metric_id, 'name': self.name, 'created_at': self.created_at, 'problem_type': self.problem_type,
                'notebook_id': self.notebook_id, 'latest_custom_metric_version': self._get_attribute_as_dict(self.latest_custom_metric_version)}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}



from .return_class import AbstractApiClass


class EmailDomain(AbstractApiClass):
    """
        Email domain (setup result, _getEmailDomainStatus, or list item in WebAppProject.email_domains).

        Args:
            client (ApiClient): An authenticated API Client instance
            emailDomainId (id): The email domain ID (null when NOT_CONFIGURED)
            hostname (str): The hostname
            verificationStatus (str): NOT_CONFIGURED | PENDING | VERIFIED | FAILED
            dnsRecords (list): DNS records (empty for list view)
            isAbacusManaged (bool): Whether DNS is Abacus-managed
    """

    def __init__(self, client, emailDomainId=None, hostname=None, verificationStatus=None, dnsRecords=None, isAbacusManaged=None):
        super().__init__(client, emailDomainId)
        self.email_domain_id = emailDomainId
        self.hostname = hostname
        self.verification_status = verificationStatus
        self.dns_records = dnsRecords
        self.is_abacus_managed = isAbacusManaged
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'email_domain_id': repr(self.email_domain_id), f'hostname': repr(self.hostname), f'verification_status': repr(
            self.verification_status), f'dns_records': repr(self.dns_records), f'is_abacus_managed': repr(self.is_abacus_managed)}
        class_name = "EmailDomain"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'email_domain_id': self.email_domain_id, 'hostname': self.hostname, 'verification_status':
                self.verification_status, 'dns_records': self.dns_records, 'is_abacus_managed': self.is_abacus_managed}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}



from .return_class import AbstractApiClass


class RegenerateLlmOption(AbstractApiClass):
    """
        An option that specifies an LLM user can regenerate with in RouteLLM.

        Args:
            client (ApiClient): An authenticated API Client instance
            name (str): The display name of the LLM (ie GPT-5)
            llmBotIcon (str): The bot icon of the LLM.
            llmName (str): The external name of the LLM (ie OPENAI_GPT5)
    """

    def __init__(self, client, name=None, llmBotIcon=None, llmName=None):
        super().__init__(client, None)
        self.name = name
        self.llm_bot_icon = llmBotIcon
        self.llm_name = llmName
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'name': repr(self.name), f'llm_bot_icon': repr(
            self.llm_bot_icon), f'llm_name': repr(self.llm_name)}
        class_name = "RegenerateLlmOption"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'name': self.name, 'llm_bot_icon': self.llm_bot_icon,
                'llm_name': self.llm_name}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}



import dataclasses

from .abstract import ApiClass
from .enums import VectorStoreTextEncoder


@dataclasses.dataclass
class VectorStoreConfig(ApiClass):
    """
    Config for indexing options of a document retriever. Default values of optional arguments are heuristically selected by the Abacus.AI platform based on the underlying data.

    Args:
        chunk_size (int): The size of text chunks in the vector store.
        chunk_overlap_fraction (float): The fraction of overlap between chunks.
        text_encoder (VectorStoreTextEncoder): Encoder used to index texts from the documents.
        chunk_size_factors (list): Chunking data with multiple sizes. The specified list of factors are used to calculate more sizes, in addition to `chunk_size`.
        score_multiplier_column (str): If provided, will use the values in this metadata column to modify the relevance score of returned chunks for all queries.
        prune_vectors (bool): Transform vectors using SVD so that the average component of vectors in the corpus are removed.
        index_metadata_columns (bool): If True, metadata columns of the FG will also be used for indexing and querying.
        use_document_summary (bool): If True, uses the summary of the document in addition to chunks of the document for indexing and querying.
        summary_instructions (str): Instructions for the LLM to generate the document summary.
        standalone_deployment (bool): If True, the document retriever will be deployed as a standalone deployment.
    """
    chunk_size: int = dataclasses.field(default=None)
    chunk_overlap_fraction: float = dataclasses.field(default=None)
    text_encoder: VectorStoreTextEncoder = dataclasses.field(default=None)
    chunk_size_factors: list = dataclasses.field(default=None)
    score_multiplier_column: str = dataclasses.field(default=None)
    prune_vectors: bool = dataclasses.field(default=None)
    index_metadata_columns: bool = dataclasses.field(default=None)
    use_document_summary: bool = dataclasses.field(default=None)
    summary_instructions: str = dataclasses.field(default=None)
    standalone_deployment: bool = dataclasses.field(default=False)


DocumentRetrieverConfig = VectorStoreConfig
